﻿-- *********************************
-- * Variable/Constant Declaration *
-- *********************************

LFMonitorCache = {score = {}, max = 2048}
-- reduce max if memory usage too high; increase if running too slowly
LFMonitor.LFM_Update_Int = 20.0
LFMonitor.CL_Update_Int = 0.5
LFMonitor.testmode = false
LFMonitor.history = {}
LFMonitor.hindex = 1
LFMonitor.TFW_PATTERN = "^_$(%d+)(.+)_$(.+)_$(.+)%$$"

-- function to deepcopy a table, from http://lua-users.org/wiki/CopyTable
function LFMonitor:deepcopy(object)

    local lookup_table = {}
    local function _copy(object)

        if type(object) ~= "table" then

            return object

        elseif lookup_table[object] then

            return lookup_table[object]

        end

        local new_table = {}
        lookup_table[object] = new_table

        for index, value in pairs(object) do

            new_table[_copy(index)] = _copy(value)

        end

        return setmetatable(new_table, getmetatable(object))

    end

    return _copy(object)

end

--concatenate two tables to produce one table
local function concatenate(t1, t2)

	local ctable = LFMonitor:deepcopy(t1)

	for _, txt in ipairs(t2) do

		table.insert(ctable, txt)

	end

	return ctable

end

--merge two or more dictionaries to produce one dictionary
local function dic_concat(...)
	local out = {}
	local i, table
	local n=select('#',...)
	for i=1,n do
		table = select(i,...)
		for key, value in pairs(table) do
			if out[key] then
				out[key] = math.max(out[key], value)
			else
				out[key] = value
			end
		end
	end
	return out
end

--build search tables
function LFMonitor:BuildTables()

	local profs = {"jc", "enc", "lw", "ins", "tlr", "bs", "alc", "eng", "misc"}
	local prof, proflist, raidlist = nil, {}, {}
	local tank, healer, melee, ranged, cc = false, false, false, false, false
	local classifierProflist, classifierClasslist, classifierRaidlist, classifierRolelist = {}, {}, {}, {}
	local p, c, rl

	for _, prof in ipairs(profs) do

		classifierProflist[prof] = LFMonitor[prof .. "txt"]

	end

	LFMonitor.proflist = proflist

	for p, c in pairs(LFMonitor.RAIDS) do
		classifierRaidlist[c.rname] = c.patt
	end

	local class, list
	for id, class in pairs(LFMonitor.classes) do
		classifierClasslist[id] = class
	end
	for id, class in pairs(LFMonitor.roles) do
		classifierRolelist[id] = class
	end

	LFMonitor.classifierProflist = classifierProflist
	LFMonitor.classifierClasslist = classifierClasslist
	LFMonitor.classifierRaidlist = classifierRaidlist
	LFMonitor.classifierRolelist = classifierRolelist
	--remove old saved data that is no longer required
	if (LFMonitorDB.craftlist) then

		LFMonitorDB.craftlist = nil

	end

	if (LFMonitorDB.options.scl) then

		LFMonitorDB.options.scl = nil

	end

end

LFMonitor.sounds = {
	[1] = {
		file = "LOOTWINDOWCOINSOUND",
		name = LFMonitor.COINS,
		type = "game"
	},
	[2] = {
		file = "INTERFACESOUND_CHARWINDOWTAB",
		name = LFMonitor.WINDOW_TAB,
		type = "game"
	},
	[3] = {
		file = "INTERFACESOUND_GAMEMENUOPEN",
		name = LFMonitor.MENU_OPENING,
		type = "game"
	},
	[4] = {
		file = "igMiniMapOpen",
		name = LFMonitor.MINIMAP_OPENING,
		type = "game"
	},
	[5] = {
		file = "igSpellBookOpen",
		name = LFMonitor.SPELLBOOK_OPENING,
		type = "game"
	},
	[6] = {
		file = "igSpellBookClose",
		name = LFMonitor.SPELLBOOK_CLOSING,
		type = "game"
	},
	[7] = {
		file = "TalentScreenOpen",
		name = LFMonitor.TALENTS_OPENING,
		type = "game"
	},
	[8] = {
		file = "igQuestLogOpen",
		name = LFMonitor.QUEST_LOG_OPENING,
		type = "game"
	},
	[9] = {
		file = "Fishing Reel in",
		name = LFMonitor.FISHING,
		type = "game"
	},
	[10]= {
		file = "HumanExploration",
		name = LFMonitor.DISCOVERY,
		type = "game"
	},
	[11]= {
		file = "applause.ogg",
		name = LFMonitor.APPLAUSE,
		type = "addon"
	},
	[12]= {
		file = "BEEPMULT.ogg",
		name = LFMonitor.BEEP,
		type = "addon"
	},
	[13]= {
		file = "CLICK10B.ogg",
		name = LFMonitor.CLICK,
		type = "addon"
	},
	[14]= {
		file = "dogbrk.ogg",
		name = LFMonitor.DOG_BARKING,
		type = "addon"
	},
	[15]= {
		file = "pcmouseclick1.ogg",
		name = LFMonitor.MOUSE_CLICK,
		type = "addon"
	},
	[16]= {
		file = "stapler2.ogg",
		name = LFMonitor.STAPLER,
		type = "addon"
	}
}

LFMonitor.fonts = {
	[1] = {
		file = "Fonts\\FRIZQT__.TTF",
		name = LFMonitor.FONT_FRIZ
	},
	[2] = {
		file = "Fonts\\ARIALN.TTF",
		name = LFMonitor.FONT_ARIALN
	},
	[3] = {
		file = "Fonts\\skurri.ttf",
		name = LFMonitor.FONT_SKURRI
	},
	[4] = {
		file = "Fonts\\MORPHEUS.ttf",
		name = LFMonitor.FONT_MORPHEUS
	}
}

LFMonitor.timestamps = {
	[1] = {
		name = LFMonitor.TS_12HR1,
		format = "%I:%M"
	},
	[2] = {
		name = LFMonitor.TS_12HR2,
		format = "%I:%M:%S"
	},
	[3] = {
		name = LFMonitor.TS_12HR3,
		format = "%I:%M %p"
	},
	[4] = {
		name = LFMonitor.TS_12HR4,
		format = "%I:%M:%S %p"
	},
	[5] = {
		name = LFMonitor.TS_24HR1,
		format = "%H:%M"
	},
	[6] = {
		name = LFMonitor.TS_24HR2,
		format = "%H:%M:%S"
	}
}

LFMonitor.defaults = {
    professions = {
		jc = 0,
		enc = 0,
		lw = 0,
		ins = 0,
		tlr = 0,
		bs = 0,
		alc = 0,
		eng = 0
	},
	monitoring = {
		lf = 0,
		wtb = 0,
		lfm = 0,
		lfg = 0,
		misc = 0,
		lfw = 0,
		wts = 0
	},
	responses = {
		jc = LFMonitor.WHISPER_ME,
		enc = LFMonitor.WHISPER_ME,
		lw = LFMonitor.WHISPER_ME,
		ins = LFMonitor.WHISPER_ME,
		tlr = LFMonitor.WHISPER_ME,
		bs = LFMonitor.WHISPER_ME,
		alc = LFMonitor.WHISPER_ME,
		eng = LFMonitor.WHISPER_ME,
		lfg = LFMonitor.LFG_RESPONSE
	},
	options = {
		scl = 1,
		hinst = 1,
		rested = 0,
		combat = 0,
		soundon = 1,
		sound = 15,
		locked = 0,
		disabled = 0,
		trade = 1,
		timestamp = 5,
		basic = "",
		adv = "",
		basicesc = "",
		useclasscolours = 0,
		overrideraid = 0
	},
	craftlist = {
		jc = nil,
		enc = nil,
		lw = nil,
		ins = nil,
		tlr = nil,
		bs = nil,
		alc = nil
	},
	classes = {
		dk = 1,
		dr = 1,
		hu = 1,
		ma = 1,
		pa = 1,
		pr = 1,
		ro = 1,
		sh = 1,
		wa = 1,
		wr = 1
	},
	roles = {
		tank = 1,
		healer = 1,
		melee = 1,
		ranged = 1,
		cc = 1,
		all = 1
	},
	channels = {
		general = 1,
		trade = 1,
		guild = 0
	},
	alert = {
		font = {
			index = 1,
			height = 14,
			timestamp = "ffffd200",
			player = "ff22ee55",
			text = "ffffffff"
		},
		backdrop = {
			background = "80000000",
			border = "ff87bdff"
		},
		position = {
			x = -1,
			y = -1,
			width = 575,
			height = 100
		}
	},
	raids = {
	}
}

LFMonitorDB = LFMonitor:deepcopy(LFMonitor.defaults)

-- Popup dialogues
StaticPopupDialogs["LFMONITOR_NOAUTOREPLY"] = {
	text = LFMonitor.CANNOT_SEND,
	button1 = _G.OKAY,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
}

StaticPopupDialogs["LFMONITOR_DEFAULTS"] = {
	text = LFMonitor.RESET_CONFIRM,
	button1 = _G.YES,
	button2 = _G.NO,
	OnAccept = function() LFMonitor:ResponseOptionsDefault() end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
}

--no operation used for certain request types
LFMonitor.nop = {""}
